@@ -1,14 +1,10 @@
 str_about = '''
-    The action module provides functionality to run individual
-    plugins as well as "pipelines" of plugins.
+    Module that provides the several "action" abstracted classes --
 
-    This module is the contact "surface" between dypi and a CUBE
-    instance. Control/manipulation of the ChRIS instance is effected
-    by a set of CLI scripts that this module creates and then executes.
-
-    NOTE: This module is "fragily" dependent on python-chrisclient and
-    caw! Changes in those modules could break things here rather
-    completely.
+    Essentially this provides a base class for CLI operations that
+    sets up a jobber object, as well some derived classes that are
+    used to run specific CLI apps as well as some API calling in
+    the larger context of CUBE/ChRIS
 '''
 
 from    .                       import  jobber
@@ -60,18 +56,27 @@ class Shexec:
         str_clean   = str_clean.strip()
         return str_clean
 
-    def docker_pull(self) -> dict:
+    def cmd_checkResponse(self, d_resp : dict) -> bool:
         """
-        Pull the container image.
+        Check the d_resp from a jobber call,
+        log a response, and return True/False
+
+        Args:
+            d_resp (dict): a response object from a jobber call
 
         Returns:
-            dict: results from jobber call
+            bool: True/False pending d_resp
         """
-        str_cmd         : str   = "docker pull %s" % self.options.dock_image
-        d_dockerpull    : dict  = self.shell.job_run(str_cmd)
-        self.env.INFO("\n%s" % str_cmd)
-        return d_dockerpull
-
+        b_OK    : bool  = False
+        if not d_resp.get('returncode'):
+            self.env.INFO("\t<green>OK!</green>")
+            self.env.INFO("\n%s" % json.dumps(d_resp, indent = 4), level = 2)
+            b_OK        = True
+        else:
+            self.env.ERROR("\tFailed!", level = 1)
+            self.env.ERROR("\n%s" % json.dumps(d_resp, indent = 4), level = 2)
+            b_OK        = False
+        return b_OK
 
     def __call__(self) ->dict:
         '''
@@ -102,20 +107,6 @@ class PluginRep(Shexec):
             'onCUBE':  json.dumps(self.env.CUBE.onCUBE())
         }
 
-    def string_clean(self, str_string : str) -> str:
-        """
-        Clean/strip/whitespace in a string
-
-        Args:
-            str_string (str): string to clean
-
-        Returns:
-            str: cleaned up string
-        """
-        str_clean   = re.sub(r';\n.*--', ';--', str_string)
-        str_clean   = str_clean.strip()
-        return str_clean
-
     def chris_plugin_info_args(self, str_containerspec : str) -> dict:
         """
         Args needed to determine json rep for chris template style plugins.
@@ -213,7 +204,7 @@ class PluginRep(Shexec):
         """
         str_cmd         : str   = "docker pull %s" % self.options.dock_image
         d_dockerpull    : dict  = self.shell.job_run(str_cmd)
-        self.env.INFO("\n%s" % str_cmd)
+        self.env.INFO("\t$> %s" % str_cmd)
         return d_dockerpull
 
     def jsonScript_buildAndExec(self, argfunc) -> dict:
@@ -235,7 +226,7 @@ class PluginRep(Shexec):
         b_status            : bool  = False
         d_json              : dict  = {}
 
-        self.env.INFO("\n%s" % str_PLCmd)
+        self.env.INFO("\t$> %s" % str_PLCmd)
         with open(str_PLCmdfile, 'w') as f:
             f.write('#!/bin/bash\n')
             f.write(str_PLCmd)
@@ -256,14 +247,15 @@ class PluginRep(Shexec):
             'stdout'        : '',
             'returncode'    : 0
         }
-        self.env.INFO("Reading JSON representation from file %s..." % self.options.json)
+        self.env.INFO("Reading JSON representation from file '%s'..." % self.options.jsonFile)
         try:
             with open(self.env.options.json, 'r') as f:
                 d_json  = json.load(f)
             d_ret['stdout']     = json.dumps(d_json, indent = 4)
         except:
-            d_ret['stderr']     = 'An error in reading the file was raised'
+            d_ret['stderr']     = "An error in reading the file '%s' was raised" % self.options.jsonFile
             d_ret['returncode'] = 1
+        self.cmd_checkResponse(d_ret)
         return d_ret
 
     def __call__(self) ->dict:
@@ -280,18 +272,125 @@ class PluginRep(Shexec):
         * failing that, using the cookiecutter calling spec
 
         '''
-        b_status        : bool  = False
-        d_runCMDresp    : dict  = {'returncode' : 1}
-        d_json          : dict  = {'error' : 'all attempts to determine representation failed'}
-        if len(self.env.options.json):
-            d_runCMDresp        = self.json_readFromFile()
-        if d_runCMDresp['returncode']:
-            self.env.INFO('\n%s' % json.dumps(self.docker_pull(), indent = 4))
+
+        def docker_pullIfNeeded() -> bool:
+            """
+            Pull the docker image -- this is needed only if we are not
+            reading the representation from file.
+
+            Returns:
+                bool: True/False pending pull success/fail
+            """
+            b_pullOK        : bool  = True
+            d_dockerPull    : dict  = {
+                "message"   : "no pull needed since user specified readFromJSONFile"
+            }
+            if not len(self.options.jsonFile):
+                d_dockerPull    = self.docker_pull()
+                b_pullOK        = self.cmd_checkResponse(d_dockerPull)
+            return d_dockerPull, b_pullOK
+
+        def jsonRepFromImage_get() -> dict:
             for argfunc in [self.chris_plugin_info_args, self.chris_cookiecutter_info_args]:
                 d_runCMDresp    = self.jsonScript_buildAndExec(argfunc)
-                if d_runCMDresp['returncode']:
-                    self.env.ERROR('\n%s' % json.dumps(d_runCMDresp, indent = 4))
-                else: break
+                if self.cmd_checkResponse(d_runCMDresp): break
+
+            return d_runCMDresp
+
+        def jsonRep_get() -> dict:
+            b_jsonOK         : bool = False
+            if len(self.env.options.jsonFile):
+                d_runCMDresp        = self.json_readFromFile()
+            else:
+                d_runCMDresp        = jsonRepFromImage_get()
+            if not d_runCMDresp['returncode']:
+                b_jsonOK    = True
+            else:
+                b_jsonOK    = False
+            return d_runCMDresp, b_jsonOK
+
+        b_status        : bool  = False
+        b_pullOK        : bool  = False
+        b_jsonOK        : bool  = False
+        d_runCMDresp    : dict  = {'returncode' : 1}
+        d_json          : dict  = {'error' : 'could not pull image'}
+
+        d_runCMDresp, b_pullOK  = docker_pullIfNeeded()
+        if b_pullOK:
+            d_runCMDresp, b_jsonOK  = jsonRep_get()
+        if not d_runCMDresp['returncode']:
+            b_status            = True
+            self.l_runCMDresp.append(d_runCMDresp)
+            try:
+                d_json          = json.loads(d_runCMDresp['stdout'])
+            except:
+                d_json          = {'error' : 'could not parse resultant stdout'}
+        return {
+            'status'            : b_status,
+            'run'               : d_runCMDresp,
+            'rep'               : d_json
+        }
+
+class CHRS(Shexec):
+    '''
+    A specialization of Shexec that runs CHRS
+    '''
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+    def login(self) -> dict:
+        """
+        Log in to CUBE via `chrs`.
+
+        Returns:
+            dict: results from login call.
+        """
+        str_cmd         : str   = self.string_clean("""
+        chrs login --address %s --username %s --password %s
+        """ % (
+                self.options.CUBEurl,
+                self.options.CUBEuser,
+                self.options.CUBEpasswd
+        ))
+        d_login         : dict  = self.shell.job_run(str_cmd)
+        self.env.INFO("\t$> %s" % str_cmd)
+        return d_login
+
+    def pipeline_add(self, str_filename : str) -> dict:
+        """
+        Add a pipeline file
+
+        Args:
+            str_filename (str): name of the pipeline file
+
+        Returns:
+            dict: results from pipeline addition
+        """
+        str_cmd         : str   = self.string_clean("""
+        chrs pipeline-file add %s
+        """ % (str_filename))
+        d_pipeline_add  : dict  = self.shell.job_run(str_cmd)
+        self.env.INFO("\t$> %s" % str_cmd)
+        return d_pipeline_add
+
+    def chrispl_onCUBEargs(self):
+        '''
+        Return a string specifying the CUBE instance
+        '''
+        return {
+            'onCUBE':  json.dumps(self.env.CUBE.onCUBE())
+        }
+
+    def __call__(self) ->dict:
+        '''
+        Entry point for adding pipelines and additionally registering
+        any plugin dependencies.
+
+        '''
+        b_status        : bool  = False
+        d_runCMDresp    : dict  = {'returncode' : 1}
+        d_json          : dict  = {'error' : 'all attempts to add pipeline failed'}
+
         if not d_runCMDresp['returncode']:
             b_status            = True
             self.l_runCMDresp.append(d_runCMDresp)
@@ -306,7 +405,7 @@ class PluginRep(Shexec):
         }
 
 
-class Register:
+class Register(Shexec):
     '''
     A class to connect to a CUBE and facilitate plugin registration
     '''
@@ -319,7 +418,7 @@ class Register:
             if k == 'env'               : self.env                  = v
             if k == 'options'           : self.options              = v
 
-        self.env.INFO("Connecting to CUBE and creating client object...")
+        self.env.INFO("Connecting to CUBE and creating client object...", level =2)
         self.cl         : client.Client = client.Client(
                                             self.env.CUBE.url,
                                             self.env.CUBE.user,
@@ -337,13 +436,19 @@ class Register:
         """
         self.env.INFO("Communicating with CUBE to register plugin...")
         try:
-            d_response = self.cl.admin_upload_plugin(self.options.computenames, d_jsonRep)
-            d_response['status']    = True
+            d_apicall   = self.cl.admin_upload_plugin(self.options.computenames, d_jsonRep)
+            d_response  = {
+                'status'        : True,
+                'stdout'        : d_apicall,
+                'stderr'        : '',
+                'returncode'    : 0
+            }
         except Exception as e:
-            d_response = {
-                'status'    : False,
-                'message'   : 'an exception occurred',
-                'exception' : str(e)
+            d_response  = {
+                'status'        : False,
+                'stdout'        : 'an exception occurred',
+                'stderr'        : str(e),
+                'returncode'    : 1
             }
         return d_response
 
@@ -390,9 +495,15 @@ class Register:
                                     d_cli['dock_image'])
             return d_jrep, b_status
         d_register  : dict  = {
-            'status'    : False
+            'status'    : False,
+            'obj'       : {
+                'run'   : {
+                    'stderr'    : ""
+                }
+            }
         }
         b_statusAND : bool  = True
+        b_status    : bool  = False
         if d_jsonRep['status']:
             d_cli   = vars(self.options)
             d_json  = d_jsonRep['rep']
@@ -400,19 +511,18 @@ class Register:
                 d_json, status = assign_if_defined(f, d_cli, d_json)
                 b_statusAND &= status
             if not b_statusAND:
-                d_register['error']     = 'A required CLI parameter is missing!'
+                d_register['obj']['run']['stderr']  = 'A required CLI parameter is missing!'
             else:
-                d_register              = self.register_do(d_json)
+                d_register['obj']['run']            = self.register_do(d_json)
+                self.cmd_checkResponse(d_register['obj']['run'])
         else:
-            self.env.ERROR("Some error was returned when determining the plugin json representation!")
-            self.env.ERROR('stdout: %s' % d_jsonRep['run']['stdout'])
-            self.env.ERROR('stderr: %s' % d_jsonRep['run']['stderr'])
-            self.env.ERROR('return: %s' % d_jsonRep['run']['returncode'])
             d_register = {
-                'status'    : False,
+                'status'    : b_status,
                 'message'   : 'a failure occured',
                 'obj'       : d_jsonRep
             }
+        if d_register['obj']['run'].get('status'):
+            d_register['status']    = d_register['obj']['run'].get('status')
 
         return d_register
 
