@@ -57,14 +57,15 @@ str_desc                =  DISPLAY_TITLE + """
 
 package_CLIself         = """
         --dock_image <container_name>                                           \\
-        --name <pluginNameInCUBE>                                               \\
-        --public_repo <repo_name>                                               \\
+        [--name <pluginNameInCUBE>]                                             \\
+        [--public_repo <repo_name>]                                             \\
+        [--public_repobase <basename>]                                          \\
         [--pluginexec <exec>]                                                   \\
         [--computenames <commalist,of,envs>]                                    \\
         [--CUBEurl <CUBEURL>]                                                   \\
         [--CUBEuser <user>]                                                     \\
         [--CUBEpasswd <password>]                                               \\
-        [--json <jsonRepFile>]                                                  \\
+        [--jsonFile <jsonRepFile>]                                              \\
         [--inputdir <inputdir>]                                                 \\
         [--outputdir <outputdir>]                                               \\
         [--man]                                                                 \\
@@ -81,21 +82,29 @@ package_CLIsynpsisArgs = """
         The name of the plugin container image. This is typically something like
 
                                 fnndsc/pl-someAnalysis
-        or
+                                       -- or --
                             localhost/fnndsc/pl-someAnalysis
 
+        and is a REQUIRED parameter.
+
         [--name <pluginNameInCUBE>]
         The name of the plugin within CUBE. Typically something like
         "pl-someAnalysis". If not supplied, name will be inferred from
         the <container_name>, by stripping and leading prefices and trailing
         versioning.
 
+        [--public_repobase <basename>]
+        The base URL of the plugin code, typically on github. If not specified,
+        is assumed to be
+
+                    https://github.com/FNNDSC
+
         [--public_repo <repo_name>]
         The URL of the plugin code, typically on github. This is accessed to
         find a README.[rst|md] which is used by the ChRIS UI when providing
         plugin details. If not supplied, the repo will be assumed to be
 
-                    https://github.com/FNNDSC/<pluginNameInCUBE>
+                    <public_repobase>/<pluginNameInCUBE>
 
         [--pluginexec <exec>]
         The name of the actual plugin executable within the image if this
@@ -114,7 +123,7 @@ package_CLIsynpsisArgs = """
         [--CUBEpasswd <password>] ("chris1234")
         The admin password.
 
-        [--json <jsonRepFile>]
+        [--jsonFile <jsonRepFile>]
         If provided, read the representation from <jsonRepFile> and do not
         attempt to run the plugin with docker.
 
@@ -286,7 +295,7 @@ parser.add_argument(
             help    = 'plugin executable name for cookiecutter style pluginsp'
 )
 parser.add_argument(
-            '--json',
+            '--jsonFile',
             default = '',
             help    = 'plugin JSON representation file'
 )
@@ -397,11 +406,11 @@ def main(args=None):
     Env.options     = options
     if Env_setup(options):
         Env.set_telnet_trace_if_specified()
-        print(DISPLAY_TITLE)
+        if int(options.verbosity) > 1: print(DISPLAY_TITLE)
         d_register  = plugin2cube.plugin2cube(options = options, env = Env).run()
         if d_register['status']:    retcode = 0
 
-    Env.INFO("terminating with code %d..." % retcode)
+    Env.INFO("terminating with code %d..." % retcode, level = 2)
     return retcode
 
 if __name__ == '__main__':
